local debuglevel = 0 -- 0=no output, 1=action output, 2=full verbose
--[===[@debug@
debuglevel=1
--@end-debug@]===]
local function debug(level,...)
	if not type(level) == "number" then
		if debuglevel >= 2 then
			print(level,...)
		end
	else
		if debuglevel >= level then
			print(...)
		end
	end
end
local function round(num, places)
	return tonumber(string.format("%."..(places or 0).."f",num))
end
local bw = BigWigs
local addon = CreateFrame("Frame", "CBMT")
local isInit = false
addon.nextTimer = 0
addon.timerSchedule = CreateFrame("Frame")
addon.timerSchedule:Hide()
addon.timerSchedule.timers = {}
addon.timerSchedule:SetScript("OnUpdate", function(self, elapsed)
	if addon.nextTimer <= 0 then
		if self.timers[1] then
			addon:broadcastTimer(round(self.timers[1].endTime-GetTime(),3),self.timers[1].label)
			tremove(self.timers,1)
		else
			self:Hide()
		end
	else
		addon.nextTimer = addon.nextTimer-elapsed
	end
end)
function addon:broadcastTimer(duration, label)
	self.nextTimer = 2
	debug(1, "Broadcasting timer '"..label.."' with duration "..duration.." sec")
	SendAddonMessage("BigWigs", "BWCustomBar "..duration.." "..label, "RAID") -- bw compability
	SendAddonMessage("DBMv4-Pizza", ("%s\t%s\t%s"):format((UnitName("player")), duration, label), "RAID") -- dbm compability
	SendAddonMessage("DXE", self.libs.Serializer.Serialize(self, "AlertsRaidBar", duration, label), "RAID")
end
local function sortTimers(a,b)
	return a.endTime < b.endTime
end
function addon:createTimer(duration, label)
	if (not label) or (not duration) then return end
	debug(1,"Creating timer '"..label.."' with duration "..duration.." sec")
	if bw and (not bw:IsEnabled()) then bw:Enable() end
	if (addon.nextTimer <= 0) then
		self:broadcastTimer(duration, label)
	else
		tinsert(addon.timerSchedule.timers, {endTime=GetTime()+duration, label=label})
		table.sort(addon.timerSchedule.timers, sortTimers)
		addon.timerSchedule:Show()
	end
end
local loadstringCache = {}
local loadstringMetaCache = {}
local function loadstring(s, force)
	if (not force) and (loadstringCache[s]) then
		return loadstringCache[s], loadstringMetaCache[s]
	end
	loadstringCache[s] = _G.loadstring(s)
	if loadstringCache[s] then
		local meta = {}
		local env = setmetatable({}, meta)
		setfenv(loadstringCache[s], env)
		loadstringMetaCache[s] = meta
		return loadstringCache[s], meta
	end
	return loadstringCache[s]
end
local defaults = { global = {
	timersSpellcast={},
	timersCustom={},
	timersMobyell={},
}}

function addon:doEvents()
	self:UnregisterAllEvents()
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	self:RegisterEvent("CHAT_MSG_MONSTER_YELL")
	self:RegisterEvent("CHAT_MSG_MONSTER_WHISPER")
	self:RegisterEvent("CHAT_MSG_MONSTER_SAY")
	self:RegisterEvent("CHAT_MSG_MONSTER_PARTY")
	self:RegisterEvent("CHAT_MSG_MONSTER_EMOTE")
	for i, data in ipairs(self.opt.timersCustom) do
		for event in pairs(data.events) do
			self:RegisterEvent(event)
			debug(2, "Registering event "..event.." for custom filter #"..i)
		end
	end
end

addon:SetScript("OnEvent", function(self, event, ...)
	local arg = {...}
	if event == "ADDON_LOADED" and arg[1] == "CBMT" then
		self:UnregisterEvent("ADDON_LOADED")
		self.db = LibStub("AceDB-3.0"):New("CBMT_DB", defaults or {})
		self.opt = self.db.global
		self.libs = {
			Serializer=LibStub("AceSerializer-3.0"),
		}
		isInit = true
		self:updateOptions()
		self:doEvents()
	end
	if event == "COMBAT_LOG_EVENT_UNFILTERED" then
		if (arg[2] == "SPELL_CAST_START" or arg[2] == "SPELL_CAST_SUCCESS") then
			debug(2,"spellID is "..arg[9],"spellName is "..arg[10])
			for n, data in ipairs(self.opt.timersSpellcast) do
				debug(2,"Checking against filter", n, data.spellFilterType, data.spellID, data.spellName)
				if (data.spellFilterType == "spellID") and (arg[9] == data.spellID) or (data.spellFilterType == "spellName") and (arg[10] == data.spellName) then
					self:createTimer(data.timerDuration, data.timerName)
				end
			end
		end
	end
	if (event == "CHAT_MSG_MONSTER_YELL") or (event == "CHAT_MSG_MONSTER_SAY") or (event == "CHAT_MSG_MONSTER_EMOTE") or (event == "CHAT_MSG_MONSTER_PARTY") or (event == "CHAT_MSG_MONSTER_WHISPER") then
		debug(2,"Received monster message:", event, ...)
		for n, data in ipairs(self.opt.timersMobyell) do
			local doesMatch = true
			if data.filterSource then
				if not data.sourceNames[arg[2]] then
					doesMatch=false
				end
			end
			if data.messageFilter and (strlen(data.messageFilter) > 0) then
				if data.filterIsPattern then
					if not arg[1]:find(data.messageFilter) then
						doesMatch=false
					end
				else
					if not arg[1] == data.messageFilter then
						doesMatch=false
					end
				end
			end
			debug(2, "Filter "..n.." does"..(not doesMatch and " not" or "").." match")
			if doesMatch then
				self:createTimer(data.timerDuration, data.timerName)
			end
		end
	end
	for n, data in ipairs(self.opt.timersCustom) do
		if data.stringFilter then
			if data.events[event] then
				local func, meta = loadstring(data.stringFilter)
				meta.__index = function(t,v)
					local argNum = tonumber(v:match("arg(%d+)"))
					if argNum then
						return arg[argNum]
					else
						return _G[v]
					end
				end
				local barDuration, barLabel = func()
				if barDuration then
					debug(2,"Bar duration returned is ",barDuration, "Bar label: ", barLabel)
					if (type(barDuration) ~= "number") or (barDuration <= 0) then
						barDuration = data.timerDuration
					end
					if (type(barLabel) ~= "string") or (strlen(barLabel) == 0) then
						barLabel = data.timerName
					end
					self:createTimer(barDuration, barLabel)
				end
			end
		end
	end
end)
addon:RegisterEvent("ADDON_LOADED")